/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.cli;

// API classes
import com.rempl.api.Artifact;
import com.rempl.api.Loader;
import com.rempl.api.LoadingException;
import com.rempl.api.Logger;
import com.rempl.api.ROM;

// AETHER
import com.rempl.cli.aether.Aether;

// for file manipulations
import java.io.File;

// for file manipulations
import java.net.URL;
import java.net.URLClassLoader;

// collections
import java.util.Collection;

// JAR manipulations
import java.util.jar.JarFile;

// apache file utils
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

/**
 * Loader of an artifact into home directory of the executer.
 * @see <a href="http://maven.apache.org/ref/3.0.3/maven-repository-metadata/repository-metadata.html">Maven Descriptor Reference</a>
 */
public final class HomeLoader implements Loader {

    /**
     * Path inside JAR file of ROM.xml.
     */
    private static final String ROM_PATH = "META-INF/REMPL/rom.xml";

    /**
     * {@inheritDoc}
     */
    @Override
    public Class forName(final String name) throws LoadingException {
        try {
            return Class.forName(
                name,
                true,
                Thread.currentThread().getContextClassLoader()
            );
        } catch (ClassNotFoundException ex) {
            throw new LoadingException(ex);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ROM load(final Artifact artifact) throws LoadingException {
        if (Logger.isTraceEnabled(this)) {
            Logger.trace(this, "#load(artifact)");
        }
        // Download all necessary JAR files through AETHER. It's
        // a special instrument to manipulate with Maven-like
        // repositories.
        final Collection<File> jars =
            new Aether().resolve(artifact.toString());
        if (Logger.isDebugEnabled(this)) {
            Logger.debug(
                this,
                "#load(%s) found %d artifacts: %s",
                artifact,
                jars.size(),
                StringUtils.join(jars, ", ")
            );
        }
        // Find, unpack, and return contents of rom.xml found
        // in one of downloaded JARs.
        final String xml = this.rom(jars);
        ROM rom;
        try {
            rom = new ROM(xml);
        } catch (com.rempl.api.ROMException ex) {
            throw new LoadingException(ex);
        }
        // Add all downloaded JARs to the classpath in order
        // to enable instantiation of workers later through
        // ROM class instances.
        this.classpath(jars);
        return rom;
    }

    /**
     * Extend current classpath with the list of JARs given. We
     * need to avoid duplicate JARs in the classpath, mostly for
     * the sake of performance.
     * @param jars List of JARs
     * @throws LoadingException If anything goes wrong
     * @see #load(Artifact)
     */
    private void classpath(final Collection<File> jars)
        throws LoadingException {
        final URL[] urls = new URL[jars.size()];
        int idx = 0;
        for (File jar : jars) {
            try {
                urls[idx] = jar.toURI().toURL();
            } catch (java.net.MalformedURLException ex) {
                throw new LoadingException(ex);
            }
            idx += 1;
        }
        Thread.currentThread().setContextClassLoader(
            new URLClassLoader(
                urls,
                Thread.currentThread().getContextClassLoader()
            )
        );
        if (Logger.isDebugEnabled(this)) {
            Logger.debug(
                this,
                "#classpath(%d JARs): equals to %s",
                jars.size(),
                StringUtils.join(urls, ":")
            );
        }
    }

    /**
     * Load ROM.xml content from one of the JARs provided. We are
     * searching through the list of files, trying to get the
     * required <tt>rom.xml</tt> from any of them. If we fail to
     * find the file - exception is thrown.
     * @param jars List of JARs
     * @return Content of ROM.xml found (yes, we load it to memory)
     * @throws LoadingException If anything goes wrong
     * @see #load(Artifact)
     */
    private String rom(final Collection<File> jars)
        throws LoadingException {
        try {
            for (File jar : jars) {
                final JarFile jarFile = new JarFile(jar.getPath());
                if (jarFile.getJarEntry(this.ROM_PATH) == null) {
                    continue;
                }
                final URL url = new URL(
                    "jar:file:"
                    + jar.getPath()
                    + "!/"
                    + this.ROM_PATH
                );
                final String xml = IOUtils.toString(url.openStream());
                if (Logger.isDebugEnabled(this)) {
                    Logger.debug(
                        this,
                        "#rom(%d JARs): rom.xml loaded in %d bytes",
                        jars.size(),
                        xml.length()
                    );
                }
                return xml;
            }
        } catch (java.net.MalformedURLException ex) {
            throw new LoadingException(ex);
        } catch (java.io.IOException ex) {
            throw new LoadingException(ex);
        }
        throw new LoadingException("ROM.xml is not found");
    }

}
