/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.cli;

// for logging
import com.rempl.api.Logger;
import com.rempl.api.RuntimeProblem;

// for CLI processing
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.OptionGroup;
import org.apache.commons.cli.Options;

/**
 * Entry point of the JAR.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Main.java 42 2010-12-17 16:16:55Z yegor256@yahoo.com $
 */
public final class Main {

    /**
     * "help" options.
     */
    public static final String OPT_HELP = "help";

    /**
     * "version" options.
     */
    public static final String OPT_VERSION = "version";

    /**
     * Private ctor, to avoid instantiation of the class by
     * any outside classes.
     */
    private Main() {
        // intentionally empty
    }

    /**
     * Entry point of the entire JAR.
     * @param args List of command-line arguments
     * @todo #9 Maybe we need to implement nicer reporting in case
     *          of exception. Now we just throw a raw exception,
     *          with a full stack trace.
     */
    public static void main(final String[] args) {
        new Main().execute(args);
    }

    /**
     * Execute the processing.
     * @param args List of command-line arguments
     */
    private void execute(final String[] args) {
        CommandLine cmd;
        try {
            cmd = new GnuParser().parse(this.options(), args);
        } catch (org.apache.commons.cli.ParseException ex) {
            throw RuntimeProblem.make(ex);
        }
        if (cmd.hasOption(this.OPT_HELP)) {
            Logger.info(this, new Help().make(this.options()));
        } else if (cmd.hasOption(this.OPT_VERSION)) {
            Logger.info(this, new Version().get());
        } else {
            final Request request = new Request(cmd);
            for (String command : cmd.getArgs()) {
                if ("read".equals(command)) {
                    request.read();
                } else if ("report".equals(command)) {
                    request.report();
                } else {
                    throw RuntimeProblem.make(
                        "Invalid command: '%s'",
                        command
                    );
                }
            }
        }
    }

    /**
     * Pre-configure all commmand line options.
     * @return The options
     * @see #execute(String[])
     * @todo #9 We need more descriptions of every options. Now they are
     *          very short and not informative. Moreover, we need to give
     *          a link to online help.
     */
    private Options options() {
        final Options options = new Options();
        final OptionGroup grp = new OptionGroup();
        grp.addOption(
            OptionBuilder
                .withDescription("get usage instructions")
                .create(this.OPT_HELP)
        );
        grp.addOption(
            OptionBuilder
                .withDescription("get version")
                .create(this.OPT_VERSION)
        );
        grp.addOption(
            OptionBuilder.withArgName("xml-file")
                .hasArg()
                .isRequired(true)
                .withDescription("absolute file name of ROM.xml")
                .create(Request.OPT_ROM)
        );
        options.addOptionGroup(grp);
        options.addOption(
            OptionBuilder.withArgName("xmi-file")
                .hasArg()
                .withDescription("XMI file name to save meta-model into")
                .create(Request.OPT_XMI)
        );
        options.addOption(
            OptionBuilder.withArgName("dir")
                .hasArg()
                .withDescription("directory where to save reports")
                .create(Request.OPT_DIR)
        );
        return options;
    }

}
